#### Data management for the Shiny application: tdbesc ####

### INRAE : Projet ReForest
### Auteur : DUMBO Marie
### Date : 28/07/2020
### Mise à jours :
### Modification 1 par Anders Mårell: 15/09/2020, add error message if reshape2 is not installed

#---------- Fonctions ----------

#' Mise en forme de donnees sylvo-cynegetique
#'
#' Cette fonction met en forme les tableaux de donnees sylvo-cynegetique en array multidimension
#'
#' @param data data.frame melt selon le package reshape2
#' @param valeur string designant la colonne de valeurs
#' @param colonne string de longueur minimale de deux, represnetant des noms de colonnes
#' @return Un array multidimension
#'
data.process <- function(data, valeur, colonne){
  if (!requireNamespace("reshape2", quietly = TRUE)) {
    stop("Package \"reshape2\" needed for this function to work. Please install it.",
         call. = FALSE)
  }
  df.data <- data[,c(valeur,colonne)]
  for (i in colnames(df.data)[-1]){
    df.data[,i] <- as.factor(df.data[[i]])
  }
  a.data <- reshape2::acast(df.data, paste0(colonne, collapse = "~"), value.var =valeur,fill = 0, drop = FALSE, fun.aggregate = sum)
  print(dim(a.data))
  names(dimnames(a.data)) <- colonne
  return(a.data)
}


#' Ajout de donnees à un ensemble
#'
#' Cette fonction ajoute des donnees à un ensemble existant
#'
#' @param data donnees sous forme d'array ayant les mêmes noms de dimension que dataset
#' @param dataset array nomme
#' @param dimension string designant la dimension où il faut ajouter la donnees
#' @return Un array multidimension
#'
add.data <- function(data, dataset, dimension){
  dim.data <- length(dim(dataset))
  if(!setequal(names(dimnames(data)), names(dimnames(dataset)))){
    stop("Les noms de dimension sont differents entre les donnees et le dataset")
  }
  if(!setequal(lapply(dimnames(dataset)[-which(names(dimnames(dataset)) == dimension)],length),
               lapply(dimnames(data)[-which(names(dimnames(dataset)) == dimension)], length))){
    stop("Les dimension de data sont different des dimension de dataset")
  }
  if( !all(names(dimnames(data)) == names(dimnames(data))) | names(dimnames(data))[dim.data] != dimension ){
    nom_dimension <- names(dimnames(data))
    nom_dimension <- nom_dimension[-which(nom_dimension == dimension)]
    nom_dimension_trie <- c(nom_dimension, dimension)
    data <- aperm(data, nom_dimension_trie)
    dataset <- aperm(dataset, nom_dimension_trie)
  }
  dim.data.fin <- dim(dataset)
  dim.data.fin[dim.data] <- dim.data.fin[dim.data] + dim(data)[dim.data]
  dimnames.data.fin <- list()
  for (i in 1:dim.data-1){
    dimnames.data.fin <- append(dimnames.data.fin, dimnames(dataset)[i])
  }
  dimnames.data.fin <- append(dimnames.data.fin, mapply(c, dimnames(dataset)[dim.data],dimnames(data)[dim.data], SIMPLIFY= FALSE))
  a.data <- array(c(dataset,data),dim = dim.data.fin, dimnames = dimnames.data.fin)
  return(a.data)
}


#' Ajout de donnees à un ensemble
#'
#' Cette fonction ajoute des donnees à un ensemble existant
#'
#' @param data1 array nomme
#' @param data2 array nomme
#' @param nom.dim string designant le nom de la dimension, par default c'est "new_dimension
#' @param nom.var string de longueur egale à 2 designant le nom des variables, par default c'est c(var1,var2)
#' @details # Les ensembles de donnees doivent avoir les mêmes dimensions et les mêmes noms de dimension
#' @return Un array multidimension
#'
add.dimension <- function(data1, data2, nom.dim="new_dimension", nom.var=c("var1","var2")){
  if(!setequal(names(dimnames(data1)), names(dimnames(data2)))){
    stop("Les noms de dimension sont differents")
  }
  if(!setequal(lapply(dimnames(data1),length), lapply(dimnames(data2), length))){
    stop("Les dimension de data1 et data2 sont differentes")
  }
  if( !all(names(dimnames(data1)) == names(dimnames(data2)))){
    data2 <- aperm(data2, names(dimnames(data1)))
  }
  nom.dimension <- list(nom.var)
  names(nom.dimension) <- nom.dim
  a.data <- array(c(data1,data2),dim = c(dim(data1),2), dimnames = c(dimnames(data1), nom.dimension))
  return(a.data)
}
